/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.*
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.model.*
import com.kakao.sdk.partner.friend.util.ChatProfileImageMaker
import com.kakao.sdk.partner.friend.view.*

/**
 * @suppress
 */
internal object PickerManager {
    @JvmStatic
    fun startActivity(
        isSingle: Boolean = false,
        isPopup: Boolean = false,
        context: Context, param: PickerFriendRequestParams,
        callback: (selectedUsers: SelectedUsers?, error: Throwable?) -> Unit
    ) {
        val exception = isParameterCorrect(param)
        val resultReceiver = resultReceiver("***** Friend Picker RESULT:", callback)
        if (!isSingle && exception != null) {
            resultReceiver.send(Activity.RESULT_CANCELED, Bundle().apply {
                putSerializable(Constants.KEY_EXCEPTION, exception)
            })
            return
        }

        Repository.loadFriendsData(
            param.pickerFriendOrder,
            param.pickerFriendType ?: Constants.DEFAULT_PICKER_FRIEND_TYPE,
            param.pickerFriendFilter ?: Constants.DEFAULT_PICKER_FRIEND_FILTER,
            param.countryCodes
        ) { error ->
            if (error != null) {
                Repository.clearData()
                callback(null, error)
            } else {
                val activity =
                    if (isPopup) FriendPopupPickerActivity::class.java else FriendPickerActivity::class.java
                val intent =
                    Intent(context, activity).putExtra(Constants.KEY_BUNDLE, Bundle().apply {
                        putBoolean(Constants.KEY_IS_SINGLE, isSingle)
                        putBoolean(Constants.KEY_IS_POPUP, isPopup)
                        putParcelable(Constants.KEY_RESULT_RECEIVER, resultReceiver)
                        putParcelable(Constants.KEY_FRIEND_PICKER_PARAM, param)
                    }).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                context.startActivity(intent)
            }
        }
    }

    @JvmStatic
    fun startActivityForChat(
        isPopup: Boolean = false,
        param: PickerChatRequestParams,
        context: Context,
        callback: (selectedUsers: SelectedUsers?, selectedChat: SelectedChat?, error: Throwable?) -> Unit
    ) {
        val filter = setPickerChatFilter(param)
        Repository.loadChatsData(filter) { error ->
            if (error != null) {
                Repository.clearData()
                callback(null, null, error)
            } else {
                val activity =
                    if (isPopup) ChatPopupPickerActivity::class.java else ChatPickerActivity::class.java
                val intent =
                    Intent(context, activity).putExtra(Constants.KEY_BUNDLE, Bundle().apply {
                        putBoolean(Constants.KEY_IS_POPUP, isPopup)
                        putParcelable(
                            Constants.KEY_RESULT_RECEIVER,
                            resultReceiver("***** Chat Picker RESULT:", callback)
                        )
                        putParcelable(Constants.KEY_CHAT_PICKER_PARAM, param)
                    }).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                context.startActivity(intent)
            }
        }
    }

    // TODO: param 받는 방식이 수정될 수 있음
    @JvmStatic
    fun startActivity(
        isSingle: Boolean = false,
        isPopup: Boolean = false,
        context: Context,
        friendParam: PickerFriendRequestParams,
        chatParam: PickerChatRequestParams,
        callback: (selectedUsers: SelectedUsers?, selectedChat: SelectedChat?, error: Throwable?) -> Unit
    ) {
        val resultReceiver = resultReceiver("***** Tab Picker RESULT:", callback)

        // TODO: 탭 피커 param이 변경되면 해당 로직 수정되야함
        for (exception in listOf(isParameterCorrect(friendParam), isParameterCorrect(chatParam))) {
            if (exception != null) {
                resultReceiver.send(Activity.RESULT_CANCELED, Bundle().apply {
                    putSerializable(Constants.KEY_EXCEPTION, exception)
                })
                return
            }
        }

        Repository.loadFriendsData(
            friendParam.pickerFriendOrder,
            friendParam.pickerFriendType ?: Constants.DEFAULT_PICKER_FRIEND_TYPE,
            friendParam.pickerFriendFilter ?: Constants.DEFAULT_PICKER_FRIEND_FILTER,
            friendParam.countryCodes
        ) { error ->
            if (error != null) {
                Repository.clearData()
                callback(null, null, error)
            } else {
                val filter = setPickerChatFilter(chatParam)
                Repository.loadChatsData(filter) { error ->
                    if (error != null) {
                        Repository.clearData()
                        callback(null, null, error)
                    } else {
                        startActivityForTab(
                            context,
                            isSingle,
                            isPopup,
                            friendParam,
                            chatParam,
                            callback
                        )
                    }
                }
            }
        }
    }

    @JvmStatic
    private fun startActivityForTab(
        context: Context,
        isSingle: Boolean,
        isPopup: Boolean,
        friendParam: PickerFriendRequestParams,
        chatParam: PickerChatRequestParams,
        callback: (selectedUsers: SelectedUsers?, selectedChat: SelectedChat?, error: Throwable?) -> Unit
    ) {
        val activity =
            if (isPopup) TabPopupPickerActivity::class.java else TabPickerActivity::class.java
        val intent =
            Intent(context, activity).putExtra(Constants.KEY_BUNDLE, Bundle().apply {
                putBoolean(Constants.KEY_IS_SINGLE, isSingle)
                putBoolean(Constants.KEY_IS_POPUP, isPopup)
                putBoolean(Constants.KEY_IS_TAB, true)
                putBoolean(Constants.KEY_IS_CHAT_MEMBER_PICKER, false)
                putParcelable(
                    Constants.KEY_RESULT_RECEIVER,
                    resultReceiver("***** Tab Picker RESULT:", callback)
                )
                putParcelable(Constants.KEY_FRIEND_PICKER_PARAM, friendParam)
                putParcelable(Constants.KEY_CHAT_PICKER_PARAM, chatParam)
            })
                .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        context.startActivity(intent)
    }

    @JvmSynthetic
    internal fun resultReceiver(
        message: String,
        callback: (selectedUsers: SelectedUsers?, error: Throwable?) -> Unit
    ): ResultReceiver = object : ResultReceiver(Handler(Looper.getMainLooper())) {
        override fun onReceiveResult(resultCode: Int, resultData: Bundle?) {
            SdkLog.d("$message $resultData")
            when (resultCode) {
                Activity.RESULT_OK -> {
                    val selectedCount = resultData?.getInt(Constants.KEY_SELECTED_TOTAL_COUNT) ?: 0
                    val friends =
                        resultData?.getParcelableArrayList<SelectedUser>(Constants.KEY_SELECTED_FRIENDS)
                    callback(friends?.let { SelectedUsers(selectedCount, it) }, null)
                }
                Activity.RESULT_CANCELED -> {
                    val error =
                        resultData?.getSerializable(Constants.KEY_EXCEPTION) as KakaoSdkError
                    callback(null, error)
                }
                else -> {
                    val error = IllegalArgumentException("Unknown resultCode")
                    callback(null, error)
                }
            }
            ChatProfileImageMaker.clearCacheImage() // Chat Profile Image 제거
            Repository.clearData() // Repository 클래스에 있는 데이터 제거
        }
    }

    @JvmSynthetic
    internal fun resultReceiver(
        message: String,
        callback: (selectedUsers: SelectedUsers?, selectedChat: SelectedChat?, error: Throwable?) -> Unit
    ): ResultReceiver = object : ResultReceiver(Handler(Looper.getMainLooper())) {
        override fun onReceiveResult(resultCode: Int, resultData: Bundle?) {
            SdkLog.d("$message $resultData")
            when (resultCode) {
                Activity.RESULT_OK -> {
                    val selectedCount = resultData?.getInt(Constants.KEY_SELECTED_TOTAL_COUNT) ?: 0
                    val friends =
                        resultData?.getParcelableArrayList<SelectedUser>(Constants.KEY_SELECTED_FRIENDS)
                    val chat = resultData?.getParcelable<SelectedChat>(Constants.KEY_SELECTED_CHAT)
                    callback(friends?.let { SelectedUsers(selectedCount, it) }, chat, null)
                }
                Activity.RESULT_CANCELED -> {
                    val error =
                        resultData?.getSerializable(Constants.KEY_EXCEPTION) as KakaoSdkError
                    callback(null, null, error)
                }
                else -> {
                    val error = IllegalArgumentException("Unknown resultCode")
                    callback(null, null, error)
                }
            }
            ChatProfileImageMaker.clearCacheImage() // Chat Profile Image 제거
            Repository.clearData() // Repository 클래스에 있는 데이터 제거
        }
    }

    @JvmStatic
    private fun isParameterCorrect(param: Parcelable): ClientError? {
        // 채팅방 멤버 피커일 때 pickerChatFilter 검사
        if (param is PickerChatRequestParams && param.selectionType == PickerChatSelectionType.CHAT_MEMBER) {
            val filter = param.pickerChatFilters ?: listOf()
            if (filter.contains(PickerChatFilter.OPEN)) {
                return ClientError(
                    ClientErrorCause.BadParameter,
                    "Chat member picker can't contain open chat member"
                )
            }
            // DIRECT 또는 MULTI에는 오픈 채팅방이 포함되어있어서 REGULAR를 같이 사용해야함.
            if ((filter.contains(PickerChatFilter.DIRECT) || filter.contains(PickerChatFilter.MULTI))
                && !filter.contains(PickerChatFilter.REGULAR)
            ) {
                return ClientError(
                    ClientErrorCause.BadParameter,
                    "'DIRECT' or 'MULTI' have to use with 'REGULAR'. because, chat member picker can't contain open chat member"
                )
            }
        }

        val minPickableCount =
            if (param is PickerFriendRequestParams) param.minPickableCount
                ?: Constants.DEFAULT_MIN_PICKABLE_COUNT
            else (param as PickerChatRequestParams).minPickableCount
                ?: Constants.DEFAULT_MIN_PICKABLE_COUNT
        val maxPickableCount =
            if (param is PickerFriendRequestParams) param.maxPickableCount
                ?: Constants.DEFAULT_MAX_PICKABLE_COUNT
            else (param as PickerChatRequestParams).maxPickableCount
                ?: Constants.DEFAULT_MAX_PICKABLE_COUNT

        return when {
            minPickableCount < Constants.MULTI_PICKER_MIN_PICKABLE_COUNT ->
                ClientError(
                    ClientErrorCause.BadParameter,
                    "Parameter minPickableCount must be greater than 1."
                )

            maxPickableCount > Constants.MULTI_PICKER_MAX_PICKABLE_COUNT ->
                ClientError(
                    ClientErrorCause.BadParameter,
                    "Parameter maxPickableCount must be 100 or less."
                )

            minPickableCount > maxPickableCount ->
                ClientError(
                    ClientErrorCause.BadParameter,
                    "Parameter maxPickableCount must be greater than or equal to parameter minPickableCount"
                )
            else -> null
        }
    }

    private fun setPickerChatFilter(param: PickerChatRequestParams): List<PickerChatFilter> {
        return when {
            param.pickerChatFilters.isNullOrEmpty() -> listOf(PickerChatFilter.REGULAR)
            else -> param.pickerChatFilters
        }
    }
}