/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.imageloader

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Handler
import android.widget.ImageView
import com.kakao.sdk.partner.friend.imageloader.ImageLoader.Companion.diskCache
import com.kakao.sdk.partner.friend.imageloader.model.DecodingResult
import com.kakao.sdk.partner.friend.imageloader.model.DownloadResult
import com.kakao.sdk.partner.friend.imageloader.model.MultiDecodingResult
import com.kakao.sdk.partner.friend.imageloader.network.ImageDownloadService
import okhttp3.Cache
import okhttp3.OkHttpClient
import okhttp3.ResponseBody
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import retrofit2.Retrofit
import java.util.Collections.synchronizedList
import java.util.concurrent.atomic.AtomicInteger

/**
 * @suppress
 */
internal class ImageDownloader(private val mainThreadHandler: Handler) {
    private val client by lazy {
        OkHttpClient.Builder().apply {
            cache(Cache(diskCache, calculateDiskCacheSize(diskCache)))
        }.build()
    }
    private val api =
        Retrofit.Builder().client(client).baseUrl(Constants.BASE_URL).build()
            .create(ImageDownloadService::class.java)

    fun downloadImage(target: ImageView, imageUrl: String) {
        val apiResult = api.downloadImage(imageUrl).execute()
        if (apiResult.isSuccessful) {
            apiResult.body()?.let {
                val bitmap = BitmapFactory.decodeStream(it.byteStream())
                val result = DownloadResult(bitmap, null, target)

                if (ImageLoader.loadCache(imageUrl) != bitmap) {
                    ImageLoader.saveCache(imageUrl, bitmap)
                }

                mainThreadHandler.run {
                    sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_COMPLETE, result))
                }
            }
            return
        }
        val result = DownloadResult(null, apiResult.message(), target)
        mainThreadHandler.run {
            sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_FAIL, result))
        }
    }

    fun downloadImage(imageUrl: String, callback: (bitmap: Bitmap?) -> Unit) {
        val apiResult = api.downloadImage(imageUrl).execute()
        val result: DecodingResult

        if (apiResult.isSuccessful) {
            apiResult.body()?.let {
                val bitmap = BitmapFactory.decodeStream(it.byteStream())
                result = DecodingResult(bitmap, null, callback)

                if (ImageLoader.loadCache(imageUrl) != bitmap) {
                    ImageLoader.saveCache(imageUrl, bitmap)
                }

                mainThreadHandler.run {
                    sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_COMPLETE, result))
                }
            }
            return
        }
        result = DecodingResult(null, apiResult.message(), callback)
        mainThreadHandler.run {
            sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_FAIL, result))
        }
    }

    fun downloadMultipleImage(
        imageUrls: List<String?>,
        callback: (bitmaps: List<Bitmap>?) -> Unit
    ) {
        var result: MultiDecodingResult
        val count = AtomicInteger(0)

        val apiResult = synchronizedList(mutableListOf<Bitmap>())

        for (url in imageUrls) {
            if (url == null) {
                count.incrementAndGet()
                if (count.get() == imageUrls.size) {
                    sendMultiDecodingResult(apiResult, callback)
                }
                continue
            }

            if (ImageLoader.loadCache(url) != null) {
                count.incrementAndGet()
                apiResult.add(ImageLoader.loadCache(url))

                if (count.get() == imageUrls.size) {
                    sendMultiDecodingResult(apiResult, callback)
                }
                continue
            }

            api.downloadImage(url).enqueue(object : Callback<ResponseBody> {
                override fun onResponse(
                    call: Call<ResponseBody>,
                    response: Response<ResponseBody>
                ) {
                    count.incrementAndGet()

                    if (response.isSuccessful) {
                        response.body()?.let {
                            val bitmap = BitmapFactory.decodeStream(it.byteStream())
                            apiResult.add(bitmap)

                            if (ImageLoader.loadCache(url) != bitmap) {
                                ImageLoader.saveCache(url, bitmap)
                            }
                            if (count.get() == imageUrls.size) {
                                sendMultiDecodingResult(apiResult, callback)
                            }
                        }
                    }
                }

                override fun onFailure(call: Call<ResponseBody>, t: Throwable) {
                    count.incrementAndGet()

                    if (count.get() == imageUrls.size) {
                        result = MultiDecodingResult(null, t.message, callback)
                        mainThreadHandler.run {
                            sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_FAIL, result))
                        }
                    }
                }
            })
        }
    }

    private fun sendMultiDecodingResult(
        apiResult: List<Bitmap>,
        callback: (bitmaps: List<Bitmap>?) -> Unit
    ) {
        val result = MultiDecodingResult(apiResult, null, callback)
        mainThreadHandler.run {
            sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_COMPLETE, result))
        }
    }
}