/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.network.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.common.json.UnknownValue
import com.kakao.sdk.common.util.KakaoJson
import kotlinx.parcelize.Parcelize

/**
 * @suppress
 * 친구 목록 조회 API 응답 클래스
 */
@Parcelize
internal data class PickerUsers<T : Parcelable>(
    @JvmSynthetic val displayAllProfile: Boolean,
    @JvmSynthetic val me: Me,
    @JvmSynthetic val friends: PickerFriends<T>
) : Parcelable

/**
 * @suppress
 */
@Parcelize
internal data class Me(
    @JvmSynthetic val id: Long,
    @JvmSynthetic val uuid: String,
    @JvmSynthetic val profileNickname: String?,
    @JvmSynthetic val profileThumbnailImage: String?
) : Parcelable

/**
 * @suppress
 * 친구 목록 조회 API 응답 클래스
 */
@Parcelize
internal data class PickerFriends<T : Parcelable>(
    @JvmSynthetic val totalCount: Int,
    @JvmSynthetic val elements: List<T>?,

    /// 조회된 친구 중 즐겨찾기에 등록된 친구 수
    @JvmSynthetic val favoriteCount: Int,

    @JvmSynthetic val beforeUrl: String?,
    @JvmSynthetic val afterUrl: String?
) : Parcelable {

    companion object {
        @JvmSynthetic
        fun <T : Parcelable> fromJson(string: String, clazz: Class<T>): PickerFriends<T> =
            KakaoJson.parameterizedFromJson(string, PickerFriends::class.java, clazz)
    }
}

/**
 * @suppress
 * 친구 피커용으로 제공되는 친구 정보입니다.
 *
 * @property id 사용자 아이디. [appRegistered]이 false 이면 null 이 됩니다.
 * @property uuid 메시지를 전송하기 위한 고유 아이디
 * @property profileNickname 친구의 대표 프로필 닉네임. 앱 가입친구의 경우 앱에서 설정한 닉네임. 미가입친구의 경우 톡 또는 스토리의 닉네임
 * @property profileThumbnailImage 친구의 썸네일 이미지 URL
 * @property favorite 즐겨찾기 추가 여부
 * @property serviceUserId 친구의 레거시(Legacy) 회원번호(Service user ID)
 * @property appRegistered 친구의 앱 연결 여부
 * @property relation 카카오톡 및 카카오스토리에서 사용자와 친구의 관계
 * @property talkOs 톡에 가입된 기기의 os 정보 (android / ios)
 * @property allowedMsg 메시지 수신이 허용되었는지 여부. 앱가입 친구의 경우는 feed msg 에 해당. 앱미가입친구는 invite msg 에 해당
 * @property ageFiltered 만 19세 이상 사용자 여부
 */
@Parcelize
internal data class PickerFriend(
    @JvmSynthetic val id: Long?,
    @JvmSynthetic val uuid: String,
    @JvmSynthetic val profileNickname: String?,
    @JvmSynthetic val profileThumbnailImage: String?,
    @JvmSynthetic val favorite: Boolean?,
    @JvmSynthetic val serviceUserId: Long?,
    @Deprecated("해당 property는 'appRegistered'로 대체되었습니다. ")
    @JvmSynthetic val registeredToApp: Boolean?,
    @JvmSynthetic val appRegistered: Boolean?,
    @JvmSynthetic val relation: PickerFriendRelation?,
    @JvmSynthetic val talkOs: String?,
    @Deprecated("해당 property는 'allowedMsg'로 대체되었습니다. ")
    @JvmSynthetic val msgAllowed: Boolean?,
    @JvmSynthetic val allowedMsg: Boolean?,
    @JvmSynthetic val ageFiltered: Boolean?
) : Parcelable

/**
 * @suppress
 */
@Parcelize
internal data class PickerFriendRelation(
    @JvmSynthetic val talk: PickerRelation,
    @JvmSynthetic val story: PickerRelation
) :
    Parcelable

/**
 * @suppress
 */
internal enum class PickerRelation {
    @SerializedName("N/A")
    @JvmSynthetic
    NONE,

    @SerializedName("friend")
    @JvmSynthetic
    FRIEND,

    @SerializedName("no_friend")
    @JvmSynthetic
    NOT_FRIEND,

    @UnknownValue
    @JvmSynthetic
    UNKNOWN;
}