/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.util

import android.content.Context
import android.graphics.*
import android.util.LruCache
import com.kakao.sdk.partner.friend.imageloader.calculateMemoryCacheSize
import com.kakao.sdk.partner.friend.view.SquircleUtils

/**
 * @suppress
 */
internal object ChatProfileImageMaker {

    val width = 110
    val height = 110
    private val shadowPaint = Paint()
    private val borderPaint = Paint()
    private val bitmapPaint = Paint()
    private lateinit var memoryCache: LruCache<Long, Bitmap>

    init {
        shadowPaint.color = Color.parseColor("#4a000000")
        shadowPaint.isAntiAlias = true

        borderPaint.color = Color.parseColor("#0c000000")
        borderPaint.isAntiAlias = true
        borderPaint.style = Paint.Style.STROKE

        bitmapPaint.isAntiAlias = true
        bitmapPaint.isFilterBitmap = true
    }

    fun getCacheImage(context: Context, id : Long): Bitmap? {
        if (!::memoryCache.isInitialized) {
            memoryCache = LruCache(calculateMemoryCacheSize(context))
        }
        return memoryCache.get(id)
    }

    fun clearCacheImage() {
        if (::memoryCache.isInitialized) {
            memoryCache.evictAll()
        }
    }

    fun makeProfileImage(context: Context, id : Long, profileImages: MutableList<Bitmap?>): Bitmap {
        val canvas: Canvas
        val paddingLeft = 1
        val paddingTop = 1
        val borderWidth = 0f

        val mergedBitmap: Bitmap = Bitmap.createBitmap(110, 110, Bitmap.Config.ARGB_8888)
        canvas = Canvas(mergedBitmap)

        val borderPath = Path()
        borderPaint.strokeWidth = dipToPixel(context,0.5f).toFloat()

        for (i in profileImages.indices) {
            canvas.save()
            val rect = decideDrawableSize(profileImages.size, i)

            canvas.translate((rect.left + paddingLeft).toFloat(), (rect.top + paddingTop).toFloat())

            borderPath.set(
                SquircleUtils.getSquircleProfilePath(
                    getProfileChildType(
                        profileImages.size,
                        i
                    )
                )
            )
            SquircleUtils.makeScale(
                borderPath,
                rect.width()
                    .toFloat() - dipToPixelToFloat(context, borderWidth) - (paddingLeft * 2).toFloat(),
                rect.height()
                    .toFloat() - dipToPixelToFloat(context, borderWidth) - (paddingTop * 2).toFloat(),
                dipToPixelToFloat(context,borderWidth / 2f)
            )

            canvas.save()
            canvas.translate(0f, 1f)
            canvas.drawPath(borderPath, shadowPaint)
            canvas.restore()

            val matrix = Matrix()
            matrix.postScale(rect.width() / 110f, rect.width() / 110f)
            val bitmap = profileImages.get(i)

            if (bitmap != null) {
                val shader = BitmapShader(bitmap, Shader.TileMode.CLAMP, Shader.TileMode.CLAMP)
                shader.setLocalMatrix(matrix)
                bitmapPaint.shader = shader
                canvas.drawPath(borderPath, bitmapPaint)
                canvas.drawPath(borderPath, borderPaint)
                canvas.restore()
            }
        }
        memoryCache.put(id, mergedBitmap)

        return mergedBitmap
    }

    private fun dipToPixel(context: Context, dip: Float): Int {
        return (dip * getDensity(context)).toInt()
    }

    private fun getDensity(context: Context): Float {
        return context.resources.displayMetrics.density
    }

    private fun dipToPixelToFloat(context: Context, dip: Float): Float {
        return dip * getDensity(context)
    }

    private fun getProfileChildType(childCount: Int, index: Int): Int {
        when (childCount) {
            1 -> return SquircleUtils.WHOLE
            2 -> return if (index == 0) SquircleUtils.LEFT_TOP_2 else SquircleUtils.WHOLE
            3 -> return if (index == 0) SquircleUtils.WHOLE else if (index == 1) SquircleUtils.LEFT_BOTTOM_3 else SquircleUtils.RIGHT_BOTTOM_3
            4 -> return SquircleUtils.WHOLE
        }
        return SquircleUtils.WHOLE
    }

    private fun getChildSize(size: Int): Int {
        return when (size) {
            2 -> (Math.min(width, height) * 30f / 48f).toInt()
            3 -> (Math.min(width, height) * 25f / 48f).toInt()
            4 -> (Math.min(width, height) * 23f / 48f).toInt()
            else -> Math.min(width, height)
        }
    }


    private fun decideDrawableSize(cnt: Int, index: Int): Rect {
        val size = getChildSize(cnt)
        when (cnt) {
            1 -> return Rect(0, 0, width, height)
            2 -> return if (index == 0) {
                Rect(0, 0, size, size)
            } else {
                Rect(width - size, height - size, width, height)
            }
            3 -> {
                val offset = (width - size) / 2
                return when (index) {
                    0 -> Rect(offset, 0, offset + size, size)
                    1 -> Rect(0, height - size, size, height)
                    else -> Rect(width - size, height - size, width, height)
                }
            }
            4 -> return when (index) {
                0 -> Rect(0, 0, size, size)
                1 -> Rect(width - size, 0, width, size)
                2 -> Rect(0, height - size, size, height)
                else -> Rect(width - size, height - size, width, height)
            }
            else -> return Rect()
        }
    }
}