/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.view

import android.content.Intent
import android.content.res.Configuration
import android.graphics.Typeface
import android.os.Bundle
import android.os.Parcelable
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintSet
import androidx.fragment.app.Fragment
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import androidx.viewpager2.widget.ViewPager2
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayoutMediator
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.Constants
import com.kakao.sdk.partner.friend.adapter.TabAdapter
import com.kakao.sdk.partner.friend.model.PickerChatRequestParams
import com.kakao.sdk.partner.friend.model.PickerChatSelectionType
import com.kakao.sdk.partner.friend.util.LayoutUtils
import com.kakao.sdk.partner.friend.viewmodel.TabViewModel
import com.kakao.sdk.v2.partner.friend.R
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkFragmentTabBinding

/**
 * @suppress
 */
internal class TabFragment : Fragment() {
    private var _binding: KakaoSdkFragmentTabBinding? = null
    private val binding get() = _binding!!

    private lateinit var tabAdapter: TabAdapter
    private lateinit var viewModel: TabViewModel

    private lateinit var title: String
    private lateinit var selectionType: PickerChatSelectionType
    private var isPopup = false

    @JvmSynthetic
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = KakaoSdkFragmentTabBinding.inflate(inflater, container, false)
        return binding.root
    }

    @JvmSynthetic
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        tabAdapter = TabAdapter(childFragmentManager, lifecycle)

        createViewModel()
        setProperty(requireActivity().intent)
        setViewPagerListener()
        initializeView()
        observeData()
    }

    private fun setViewPagerListener() {
        binding.pager.registerOnPageChangeCallback(object : ViewPager2.OnPageChangeCallback() {
            override fun onPageSelected(position: Int) {
                viewModel.clearTab(position)
                binding.titleLayout.apply {
                    pickedCountTv.visibility =
                        if (position == FRIEND_TAB && doneBtn.isEnabled) View.VISIBLE else View.GONE
                    doneBtn.text =
                        if (position == FRIEND_TAB || selectionType == PickerChatSelectionType.CHAT) getString(
                            R.string.ok
                        ) else getString(R.string.next)
                }
                if (position == FRIEND_TAB) {
                    if (viewModel.isEmptyFriendError.value == true) {
                        setEmptyView(View.VISIBLE, getString(R.string.empty_friend_message))
                    } else {
                        setEmptyView(View.GONE)
                    }
                } else {
                    if (viewModel.isEmptyChatError.value == true) {
                        setEmptyView(View.VISIBLE, getString(R.string.empty_chat_message))
                    } else {
                        setEmptyView(View.GONE)
                    }
                }
            }
        })
    }

    private fun createViewModel() {
        viewModel = ViewModelProvider(
            requireActivity(),
            ViewModelProvider.NewInstanceFactory()
        ).get(TabViewModel::class.java)
    }

    private fun initializeView() {
        binding.apply {
            pager.offscreenPageLimit = 1
            pager.adapter = tabAdapter
            TabLayoutMediator(tabLayout, pager) { _, _ -> }.attach()

            titleLayout.apply {
                title.text = this@TabFragment.title
                doneBtn.setOnClickListener {
                    viewModel.sendResult()
                }
                backBtn.setOnClickListener {
                    viewModel.cancelPicker()
                }
            }

            friendTab.setOnClickListener {
                tabLayout.selectTab(tabLayout.getTabAt(FRIEND_TAB))
            }

            chatTab.setOnClickListener {
                tabLayout.selectTab(tabLayout.getTabAt(CHAT_TAB))
            }

            tabLayout.apply {
                addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
                    override fun onTabSelected(tab: TabLayout.Tab) {
                        updateTab(tab.position)
                    }

                    override fun onTabUnselected(tab: TabLayout.Tab) {
                    }

                    override fun onTabReselected(tab: TabLayout.Tab) {
                    }
                })
                updateTab(FRIEND_TAB)
            }
        }
        if (isPopup) {
            setPopupLayout()
        }
    }

    private fun updateTab(position: Int) {
        val selectedColor = TypedValue().apply {
            requireContext().theme.resolveAttribute(R.attr.gray900s, this, true)
        }
        val unselectedColor = TypedValue().apply {
            requireContext().theme.resolveAttribute(R.attr.gray500s, this, true)
        }
        binding.friendTab.apply {
            setTypeface(null, if (position == FRIEND_TAB) Typeface.BOLD else Typeface.NORMAL)
            setTextColor(if (position == FRIEND_TAB) selectedColor.data else unselectedColor.data)
        }
        binding.chatTab.apply {
            setTypeface(null, if (position == CHAT_TAB) Typeface.BOLD else Typeface.NORMAL)
            setTextColor(if (position == CHAT_TAB) selectedColor.data else unselectedColor.data)
        }
    }

    private fun setPopupLayout() {
        binding.apply {
            titleLayout.apply {
                val marginTop = if (LayoutUtils.orientation == Configuration.ORIENTATION_PORTRAIT) {
                    resources.getDimensionPixelSize(R.dimen.popup_title_bar_padding_top)
                } else {
                    resources.getDimensionPixelSize(R.dimen.popup_title_bar_padding_top_landscape)
                }
                root.setPadding(0, marginTop, 0, 0)
                title.apply {
                    gravity = Gravity.CENTER

                    // 뷰가 그려지기 전에 width를 계산
                    backBtn.measure(View.MeasureSpec.UNSPECIFIED, View.MeasureSpec.UNSPECIFIED)
                    doneBtnLayout.measure(
                        View.MeasureSpec.UNSPECIFIED,
                        View.MeasureSpec.UNSPECIFIED
                    )

                    // 타이틀 좌, 우 뷰 중에서 더 넓은 뷰 넓이에 마진값 적용
                    val marginHorizontal = kotlin.math.max(
                        backBtn.measuredWidth,
                        doneBtnLayout.measuredWidth
                    ) + resources.getDimensionPixelSize(R.dimen.popup_title_bar_text_margin_start)

                    ConstraintSet().apply {
                        clone(root)
                        connect(
                            id,
                            ConstraintSet.START,
                            ConstraintSet.PARENT_ID,
                            ConstraintSet.START,
                            marginHorizontal
                        )
                        connect(
                            id,
                            ConstraintSet.END,
                            ConstraintSet.PARENT_ID,
                            ConstraintSet.END, marginHorizontal
                        )
                    }.applyTo(root)
                }
            }
        }
    }

    private fun setProperty(intent: Intent) {
        try {
            intent.extras?.getBundle(Constants.KEY_BUNDLE)?.run {
                val param =
                    getParcelable<Parcelable>(Constants.KEY_CHAT_PICKER_PARAM)!! // TODO: 탭 피커용 파람으로 수정될 예정
                title = (param as PickerChatRequestParams).title
                    ?: getString(R.string.select_friend)
                isPopup = getBoolean(Constants.KEY_IS_POPUP)
                selectionType = param.selectionType
            }
        } catch (e: Throwable) {
            SdkLog.e(e)
            viewModel.cancelPicker()
        }
    }

    private fun observeData() {
        viewModel.apply {
            pickedFriendsCount.observe(viewLifecycleOwner, Observer { count ->
                binding.titleLayout.apply {
                    if (tabPosition == FRIEND_TAB) {
                        pickedCountTv.text = count.toString()
                        pickedCountTv.visibility =
                            if (count > 0) View.VISIBLE else View.GONE
                    }
                }
            })
            isDoneButtonEnabled.observe(viewLifecycleOwner, Observer {
                binding.titleLayout.doneBtn.isEnabled = it
            })
            isEmptyFriendError.observe(viewLifecycleOwner, Observer {
                if (it) {
                    setEmptyView(View.VISIBLE, getString(R.string.empty_friend_message))
                } else {
                    setEmptyView(View.GONE)
                }
            })
            isEmptyChatError.observe(viewLifecycleOwner, Observer {
                if (it) {
                    setEmptyView(View.VISIBLE, getString(R.string.empty_chat_message))
                } else {
                    setEmptyView(View.GONE)
                }
            })
        }
    }

    private fun setEmptyView(visibility: Int, message: String = "") {
        if (visibility == View.VISIBLE) {
            binding.emptyView.visibility = View.VISIBLE
            binding.emptyView.setErrorMessage(message)
        } else if (visibility == View.GONE) {
            binding.emptyView.visibility = View.GONE
        }
    }

    @JvmSynthetic
    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    companion object {
        const val FRIEND_TAB = 0
        const val CHAT_TAB = 1
    }
}